#!/usr/bin/ruby

require 'md5'
require 'escape'
require 'ostruct'


class DAK

  def initialize(base_dir, dak_utils)
    @base_dir = base_dir
    @dak_utils = dak_utils
  end

  def list_new(with_hash = false)
    list = {}
  
    new_queue = File.join(@base_dir, "queue/new")
  
    Dir.new(new_queue).each do |file|
      next unless file =~ /.*\.changes$/
  
      File.open(File.join(new_queue, file)) do |fp|
        chg_pkgname = file.split("_").first

        data = {}
	data['chg_pkgname'] = chg_pkgname
        hashstr = chg_pkgname
  
        while not fp.eof?
          line = fp.readline.strip
  	  break if line == "-----BEGIN PGP SIGNATURE-----"
  
          field, value = line.split(":", 2)
  	  next if value.nil?
  
  	  value.strip!
  	  if ["Source", "Version", "Architecture", "Distribution"].include?(field)
            data[field] = value
  	    hashstr += "/" + value
  	  end
        end
  
        data['file'] = file
        arch_list = data['Architecture'].split(" ") - ["source", "all"]
	data['upload_arch'] = (arch_list.empty?) ? "all" : arch_list.shift
        pkg_index = MD5::hexdigest(hashstr)

        list[pkg_index] = OpenStruct.new(data)
      end
    end

    if with_hash
      return list, list.keys.hash
    else
      return list
    end
  end

  def install
      system("#{File.join(@base_dir, "config/cron.accepted")} >/dev/null 2>#{File.join(@base_dir, "cron.accepted_cortex.error_log") }")
      return $? == 0
  end

  def scan
      system("#{File.join(@base_dir, "config/cron.unchecked")} >/dev/null 2>#{File.join(@base_dir, "cron.unchecked_cortex.error_log") }")
      return $? == 0
  end

  def moderate_package(pkginfo, do_accept, reason = nil)
    if do_accept
      log_cmd = "ACCEPTED"
      dak_cmd = "A"
      dak_extra = ""
    else
      log_cmd = "REJECTED"
      dak_cmd = "M"
      reason ||= "Refused without reason"
      dak_extra = "-m " + Escape.shell_command(reason).to_s
    end
  
    system("cd #{File.join(@base_dir, "queue/new")}; echo '#{dak_cmd}' | DAK_OVERRIDE=1 dak process-new #{dak_extra} #{pkginfo.file} >/dev/null 2>/dev/null")
    ret = $?
  
    if ret
      msg = Time.now.strftime("%Y-%m-%d %H:%M") + ": " + pkginfo.file + "\n"
      msg += "\n"
      msg += "was in NEW\n"
      msg += pkginfo.file + "\n"
      msg += log_cmd + " for " + pkginfo.Distribution + "\n"
      msg += "--\n"
      File.open(File.join(@base_dir, "queue/REPORT"), "a") do |fp|
        fp << msg
      end
    end
  
    return ret
  end

  def latest_version(pkg, arch="i386", suite=nil)
    ver = nil
    IO.popen("#{File.join(@dak_utils, "scripts/latest_pkg_version #{pkg} all,#{arch} #{suite}")} 2>/dev/null") do |p|
      ver = p.readline.strip unless p.eof?
    end
    return nil if ver == ""
    return ver
  end

end

